import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import _objectWithoutPropertiesLoose from "reshow-runtime/es/helpers/objectWithoutPropertiesLoose";
var _excluded = ["scrollMargin", "noDelay", "targetId", "container", "children"];
import { isValidElement, useRef } from "react";
import { useReturn } from "reshow-return";
import { build } from "react-atomic-molecule";
import { UNDEFINED, DEFAULT } from "reshow-constant";
import callfunc from "call-func";
import scrollStore from "../../src/stores/scrollStore.mjs";
import fastScrollStore from "../../src/stores/fastScrollStore.mjs";

var useScrollReceiver = function useScrollReceiver(props) {
  var _props$scrollMargin = props.scrollMargin,
      scrollMargin = _props$scrollMargin === void 0 ? DEFAULT : _props$scrollMargin,
      _props$noDelay = props.noDelay,
      noDelay = _props$noDelay === void 0 ? false : _props$noDelay,
      targetId = props.targetId,
      container = props.container,
      children = props.children,
      restProps = _objectWithoutPropertiesLoose(props, _excluded);

  var lastIsShown = useRef();
  var store = noDelay ? fastScrollStore : scrollStore;

  var _useReturn = useReturn(["scroll"], store),
      _useReturn$scroll = _useReturn.scroll,
      scrollInfo = _useReturn$scroll === void 0 ? {} : _useReturn$scroll;

  var activeId = store.getState().get("m" + scrollMargin);
  var scrollTop = scrollInfo.top;
  var pos = store.scroller.getOffset(targetId) || {};
  var isShown = lastIsShown.current || false;
  var active = UNDEFINED !== typeof targetId && targetId === activeId;

  var targetInfo = _objectSpread(_objectSpread({}, pos), {}, {
    active: active,
    scrollTop: scrollTop,
    scrollInfo: scrollInfo,
    scrollMargin: scrollMargin,
    isShown: isShown,
    targetId: targetId
  });

  if (pos.isOnScreen) {
    targetInfo.isShown = true;
    lastIsShown.current = true;
  }

  if (!isNaN(scrollMargin)) {
    store.scroller.addMargin(scrollMargin);
  }

  return {
    targetInfo: targetInfo,
    container: container,
    children: children,
    restProps: restProps
  };
};

var ScrollReceiver = function ScrollReceiver(props) {
  var _useScrollReceiver = useScrollReceiver(props),
      targetInfo = _useScrollReceiver.targetInfo,
      container = _useScrollReceiver.container,
      children = _useScrollReceiver.children,
      restProps = _useScrollReceiver.restProps;

  var nextContainer = container;
  var nextChildren = children;

  if (!nextContainer) {
    nextContainer = children;
    nextChildren = null;
  }

  if (!nextChildren && /*#__PURE__*/isValidElement(nextContainer)) {
    nextChildren = nextContainer.props.children;
  }

  return build(nextContainer)(_objectSpread(_objectSpread({}, restProps), {}, {
    targetInfo: targetInfo
  }),
  /**
   * sould not use build here, let nextContainer handle children by itself.
   */
  nextChildren);
};

ScrollReceiver.displayName = "ScrollReceiver";
export default ScrollReceiver;