import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import _objectWithoutPropertiesLoose from "reshow-runtime/es/helpers/objectWithoutPropertiesLoose";
var _excluded = ["noDelay", "monitorScroll", "attachDestRetry", "id", "scrollMargin", "children", "container", "className", "attachDest"];
import { useRef, useMemo, useState, useEffect, useCallback } from "react";
import get from "get-object-value";
import { mixClass, build, getDisplayName, SemanticUI } from "react-atomic-molecule";
import { useDebounce, useMounted } from "reshow-hooks";
import scrollStore from "../../src/stores/scrollStore.mjs";
import fastScrollStore from "../../src/stores/fastScrollStore.mjs";

var useScrollSpy = function useScrollSpy(props) {
  /**
   * monitorScroll use in store, in component just for reset props.
   */
  var _props$noDelay = props.noDelay,
      noDelay = _props$noDelay === void 0 ? false : _props$noDelay,
      _props$monitorScroll = props.monitorScroll,
      monitorScroll = _props$monitorScroll === void 0 ? true : _props$monitorScroll,
      _props$attachDestRetr = props.attachDestRetry,
      attachDestRetry = _props$attachDestRetr === void 0 ? 20 : _props$attachDestRetr,
      id = props.id,
      scrollMargin = props.scrollMargin,
      children = props.children,
      container = props.container,
      className = props.className,
      attachDest = props.attachDest,
      restProps = _objectWithoutPropertiesLoose(props, _excluded);

  var _useState = useState(id),
      targetId = _useState[0],
      setTargetId = _useState[1];

  var _mount = useMounted();

  var lastEl = useRef();
  var lastConfig = useRef({});
  lastConfig.current = _objectSpread(_objectSpread({}, lastConfig.current), {}, {
    id: targetId,
    attachDest: lastConfig.current.attachDest || attachDest,
    monitorScroll: monitorScroll,
    scrollMargin: scrollMargin
  });
  var nextContainer = useRef();
  useEffect(function () {
    var store = noDelay ? fastScrollStore : scrollStore;
    var id = store.scroller.attach(expose);
    lastConfig.current.store = store;
    setTargetId(id);
    return function () {
      store.scroller.detach(expose);
    };
  }, []);
  var warnDebounce = useDebounce(function (args) {
    // for lazy render component, that warn delay 1.5 secs.
    if (!lastEl.current && _mount()) {
      // maybe could get lastEl late.
      console.warn('Please use SemanticUI. props.container -> import {SemanticUI} from "react-atomic-molecule"', args);
    }
  }, 1500);
  var getOffsetEl = useCallback(function () {
    if (lastEl.current) {
      return lastEl.current;
    } else {
      warnDebounce({
        targetId: targetId,
        container: nextContainer.current
      });
    }
  }, [targetId]);
  var expose = {
    lastConfig: lastConfig,
    getOffsetEl: getOffsetEl,
    detach: function detach() {
      return lastConfig.current.store.scroller.detach(expose);
    },
    getId: function getId() {
      return lastConfig.current.id;
    },
    setId: setTargetId,
    getAttachDest: function getAttachDest() {
      return lastConfig.current.attachDest;
    },
    setAttachDest: function setAttachDest(attachDest) {
      return lastConfig.current.attachDest = attachDest;
    },
    getMonitorScroll: function getMonitorScroll() {
      return lastConfig.current.monitorScroll;
    },
    getScrollMargin: function getScrollMargin() {
      return lastConfig.current.scrollMargin;
    },
    getAttachDestRetry: function getAttachDestRetry() {
      return attachDestRetry;
    }
  };
  restProps.id = targetId;
  restProps.refCb = lastEl;
  restProps.className = mixClass(className, "spy-tar-" + targetId);
  return {
    targetId: targetId,
    className: className,
    children: children,
    container: container,
    noDelay: noDelay,
    nextContainer: nextContainer,
    restProps: restProps
  };
};

var ScrollSpy = function ScrollSpy(props) {
  var _useScrollSpy = useScrollSpy(props),
      targetId = _useScrollSpy.targetId,
      className = _useScrollSpy.className,
      children = _useScrollSpy.children,
      container = _useScrollSpy.container,
      noDelay = _useScrollSpy.noDelay,
      nextContainer = _useScrollSpy.nextContainer,
      restProps = _useScrollSpy.restProps;

  return useMemo(function () {
    var hasScrollReceiver = "ScrollReceiver" === getDisplayName(children) ? true : false;
    var nextProps;

    if (hasScrollReceiver) {
      nextContainer.current = children;
      nextProps = _objectSpread(_objectSpread(_objectSpread({}, children.props), restProps), {}, {
        targetId: targetId,
        container: container,
        noDelay: noDelay
      });
    } else {
      nextContainer.current = container || SemanticUI;
      nextProps = _objectSpread(_objectSpread({}, restProps), {}, {
        children: children
      });
    }

    return build(nextContainer.current)(nextProps);
  }, [children, targetId]);
};

export default ScrollSpy;