import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import _objectWithoutPropertiesLoose from "reshow-runtime/es/helpers/objectWithoutPropertiesLoose";
var _excluded = ["targetId", "scrollRefLoc", "scrollRefId", "scrollMargin", "style", "preventDefault", "noDelay", "onClick"];
import { useEffect, useState, useCallback, useRef } from "react";
import smoothScrollTo from "smooth-scroll-to";
import getOffset from "getoffset";
import callfunc from "call-func";
import { doc } from "win-doc";
import { getAnchorPath } from "reshow-url";
import { ScrollReceiver } from "../../src/index.mjs";
import scrollStore from "../../src/stores/scrollStore.mjs";
import fastScrollStore from "../../src/stores/fastScrollStore.mjs";
import { jsx as _jsx } from "react/jsx-runtime";
var scollTimer = null;

var resetTimer = function resetTimer() {
  if (scollTimer) {
    clearTimeout(scollTimer);
    scollTimer = null;
  }
};

var useSmoothScrollLink = function useSmoothScrollLink(props) {
  var targetId = props.targetId,
      _props$scrollRefLoc = props.scrollRefLoc,
      scrollRefLoc = _props$scrollRefLoc === void 0 ? "bottom" : _props$scrollRefLoc,
      _props$scrollRefId = props.scrollRefId,
      scrollRefId = _props$scrollRefId === void 0 ? "" : _props$scrollRefId,
      scrollMargin = props.scrollMargin,
      style = props.style,
      _props$preventDefault = props.preventDefault,
      preventDefault = _props$preventDefault === void 0 ? true : _props$preventDefault,
      _props$noDelay = props.noDelay,
      noDelay = _props$noDelay === void 0 ? false : _props$noDelay,
      onClick = props.onClick,
      restProps = _objectWithoutPropertiesLoose(props, _excluded);

  var _useState = useState(),
      scrollRefElement = _useState[0],
      setScrollRefElement = _useState[1];

  var scrollTo = function scrollTo(lazyScrollTime, duringTime) {
    if (lazyScrollTime === void 0) {
      lazyScrollTime = [500, 800, 500];
    }

    var offset = getStore().scroller.getOffset(targetId);

    if (offset) {
      var _margin = getMargin();

      var to = offset.top - _margin;
      smoothScrollTo(to, duringTime, null, function () {
        var nextScrollTime = lazyScrollTime.shift();

        if (null != nextScrollTime) {
          resetTimer();
          scollTimer = setTimeout(function () {
            return scrollTo(lazyScrollTime, 100);
          }, nextScrollTime);
        }
      });
    }
  };

  useEffect(function () {
    var dom = doc().getElementById(scrollRefId);

    if (dom) {
      setScrollRefElement(dom);
    }

    if (getAnchorPath().anchor.substring(1) === targetId) {
      setTimeout(function () {
        return scrollTo([500, 500, 500, 500], 100);
      });
    }

    return function () {
      resetTimer();
    };
  }, []);
  /**
   * When modify getMargin number, should also take care store isActive logic.
   *
   * https://github.com/react-atomic/organism-react-scroll-nav/blob/main/src/stores/scrollStore.js#L119
   */

  var getMargin = useCallback(function () {
    var margin = 0;

    if (scrollRefElement) {
      var refOffset = getOffset(scrollRefElement, false);

      switch (scrollRefLoc) {
        case "bottom":
          margin += refOffset.bottom - refOffset.top;
          break;

        default:
        case "top":
          break;
      }
    }

    if (!isNaN(scrollMargin)) {
      margin += scrollMargin;
    }

    margin -= 2;
    return margin;
  }, [scrollRefLoc, scrollMargin, scrollRefElement]);

  var getStore = function getStore() {
    return noDelay ? fastScrollStore : scrollStore;
  };

  var handler = {
    click: function click(e) {
      if (preventDefault) {
        e.preventDefault();
      }

      callfunc(onClick);
      scrollTo();
    }
  };
  var margin = getMargin();
  return {
    restProps: restProps,
    handler: handler,
    targetId: targetId,
    margin: margin,
    style: style
  };
};

var SmoothScrollLink = function SmoothScrollLink(props) {
  var _useSmoothScrollLink = useSmoothScrollLink(props),
      restProps = _useSmoothScrollLink.restProps,
      handler = _useSmoothScrollLink.handler,
      margin = _useSmoothScrollLink.margin,
      style = _useSmoothScrollLink.style,
      targetId = _useSmoothScrollLink.targetId;

  return /*#__PURE__*/_jsx(ScrollReceiver, _objectSpread(_objectSpread({
    atom: "a"
  }, restProps), {}, {
    targetId: targetId,
    scrollMargin: margin,
    style: _objectSpread(_objectSpread({}, Styles.link), style),
    onClick: handler.click
  }));
};

export default SmoothScrollLink;
var Styles = {
  link: {
    cursor: "pointer"
  }
};